/****************************************************************************
 *
 *  cactus.c : դκǾåȤΥɽ
 *
 *  Programmed by Shuji Nakamura  Time-stamp: <2001-07-11>
 *
 ***************************************************************************/


#include <stdio.h>      /* NULL, printf() */
#include <stdlib.h>     /* EXIT_... */
#include <math.h>       /* fabs() */
#include <float.h>      /* DBL_MAX */
#include "fheap.h"
#include "pheap.h"
#include "bheap.h"
#include "dheap.h"
#include "graph.h"
#include "slist.h"
#include "error.h"
#include "mymalloc.h"
#include "mygetopt.h"


/*  0 : դʣ򤷤ʤʵΰ O(|E|)
 * >0 : դʣ򤹤ʵΰ O(|V||E|) */
#define REPLICATE 0

/* ҡפμ
 * 'f': եܥʥåҡ
 * 'p': ڥ󥰥ҡ
 * 'd': d ҡ
 * 'b': Хʥҡ */
#define HEAP_TYPE 'f'

/* ɽ٥
 * 0 : ɽʤ
 * 1 : ̤Τɽ
 * 2 : вɽ
 * 3 : ˾ܤвɽ */
int verbose = 1;


/***************************************************************************/


#if HEAP_TYPE == 'f'
typedef FHEAP HEAP;
#define create_heap       create_fheap
#define init_heap         init_fheap
#define free_heap         free_fheap
#define find_max_heap     find_max_fheap
#define insert_heap       insert_fheap
#define delete_max_heap   delete_max_fheap
#define increase_key_heap increase_key_fheap
#elif HEAP_TYPE == 'p'
typedef PHEAP HEAP;
#define create_heap       create_pheap
#define init_heap         init_pheap
#define free_heap         free_pheap
#define find_max_heap     find_max_pheap
#define insert_heap       insert_pheap
#define delete_max_heap   delete_max_pheap
#define increase_key_heap increase_key_pheap
#elif HEAP_TYPE == 'd'
typedef DHEAP HEAP;
#define create_heap       create_dheap
#define init_heap         init_dheap
#define free_heap         free_dheap
#define find_max_heap     find_max_dheap
#define insert_heap       insert_dheap
#define delete_max_heap   delete_max_dheap
#define increase_key_heap increase_key_dheap
#elif HEAP_TYPE == 'b'
typedef BHEAP HEAP;
#define create_heap       create_bheap
#define init_heap         init_bheap
#define free_heap         free_bheap
#define find_max_heap     find_max_bheap
#define insert_heap       insert_bheap
#define delete_max_heap   delete_max_bheap
#define increase_key_heap increase_key_bheap
#endif


/***************************************************************************/


/* ܽ¤ */
typedef struct mao
{
  int n;                /*  */
  int no;               /* դο(Number of Ordering node) */
  int t1;               /* ֹܽ */
  int t2;               /* ܽ飲ܤֹ */
  int *nfo;             /* ܽФֹ(Node From Order) */
  int *ofn;             /* ֹФܽ(Order From Node) */
}
MAO;


/* ܽǡ */
void
create_mao (int n, MAO * mao)
{
  mao->n = n;
  mao->no = 0;
  mao->t1 = mao->t2 = 0;
  mao->ofn = (int *) my_malloc (n * sizeof (int));
  mao->nfo = (int *) my_malloc (n * sizeof (int));
}


/* ܽǡ */
void
free_mao (MAO * mao)
{
  my_free (mao->ofn, mao->n);
  my_free (mao->nfo, mao->n);
}


/***************************************************************************/


/* å¤ */
typedef struct stack
{
  int n;
  int top;
  int *data;
}
STACK;


void
create_stack (int n, STACK * st)
{
  st->n = n;
  st->data = my_malloc (n * sizeof (int));
  st->top = 0;
}


void
init_stack (STACK * st)
{
  st->top = 0;
}


void
free_stack (STACK * st)
{
  my_free (st->data, st->n);
}


int
pop (STACK * st)
{
  st->top--;
  return st->data[st->top];
}


void
push (int v, STACK * st)
{
  st->data[st->top] = v;
  st->top++;
}


/***************************************************************************/


#ifndef NIL
#define NIL -1
#endif


/* ޥåԥѥꥹ */
typedef struct mlist
{
  int nc;               /*  */
  int ng;               /* ϥդ */
  int *head;            /* ꥹȤƬֹ */
  int *prev, *next;     /*  */
  int *dest;            /* ϥդΥޥåԥݻ(DESTination) */
}
MLIST;


void
create_mlist (int nc, int ng, MLIST * ml)
{
  int i;

  ml->nc = nc;
  ml->ng = ng;
  ml->head = my_malloc (nc * sizeof (int));
  ml->prev = my_malloc (ng * sizeof (int));
  ml->next = my_malloc (ng * sizeof (int));
  ml->dest = my_malloc (ng * sizeof (int));

  for (i = 0; i < nc; i++)
    ml->head[i] = NIL;
  for (i = 0; i < ng; i++)
    {
      ml->next[i] = i;
      ml->prev[i] = ml->dest[i] = NIL;
    }
}


void
free_mlist (MLIST * ml)
{
  my_free (ml->head, ml->nc);
  my_free (ml->prev, ml->ng);
  my_free (ml->next, ml->ng);
  my_free (ml->dest, ml->ng);
}


void
insert_into_mlist (int v, int u, MLIST * ml)
{
  if (ml->next[v] != v)
    return;

  ml->next[v] = ml->head[u];
  ml->prev[v] = NIL;
  ml->head[u] = v;
  if (ml->next[v] != NIL)
    ml->prev[ml->next[v]] = v;
}


void
delete_from_mlist (int v, MLIST * ml)
{
  if (ml->next[v] == v)
    return;

  if (ml->next[v] != NIL)
    ml->prev[ml->next[v]] = ml->prev[v];
  if (ml->prev[v] != NIL)
    ml->next[ml->prev[v]] = ml->next[v];
  else
    ml->head[ml->dest[v]] = ml->next[v];
  ml->next[v] = v;
}


int
get_member_in_mlist (int u, int nbuf[], MLIST * ml)
{
  int nn, v;

  v = ml->head[u];
  for (nn = 0; v != NIL; nn++)
    {
      nbuf[nn] = v;
      v = ml->next[v];
    }
  return nn;
}


void
move_mlist (int s, int t, MLIST * ml)
{
  int v;

  for (v = ml->head[s]; v != NIL; v = ml->next[v])
    ml->dest[v] = t;
  ml->head[t] = ml->head[s];
  ml->head[s] = NIL;
}


/***************************************************************************/


/* ܽ(Maximum Adjacency Ordering)
 * s : ׻ֹ
 * gc : ǽդչ¤ΤؤΥݥ
 * mao : ܽ¤ΤؤΥݥ
 * heap : ҡ׹¤ΤؤΥݥ */
void
get_mao (int s, GRAPH_C * gc, MAO * mao, HEAP * heap)
{
  int i, v;
  EDGE_C *e;

  /* v ʳˤҡ׺ */
  init_heap (heap);
  for (v = gc->ehead; v != NIL; v = gc->next[v])
    {
      if (v != s)
        insert_heap (v, 0, heap);
      mao->ofn[v] = NIL; /* MAO ̤ʤ mao->ofn[v]  NIL */
    }

  mao->nfo[0] = s;
  mao->ofn[s] = 0;

  /* ܽ׻ */
  v = s;
  for (i = 1; i < gc->ne; i++)
    {
      for (e = gc->adjlist[v]; e != NULL; e = e->next)
        {
          /* Ťʬ */
          if (mao->ofn[e->node] == NIL)
            increase_key_heap (e->node, e->weight, heap);
        }
      v = find_max_heap (heap);
      delete_max_heap (heap);
      mao->nfo[i] = v;
      mao->ofn[v] = i;
    }

  mao->no = gc->ne;
  mao->t1 = mao->nfo[mao->no - 1];
  mao->t2 = mao->nfo[mao->no - 2];
}


/* Ǿåȼ
 * gc : ǽդչ¤ΤؤΥݥ
 * mao : ܽ¤ΤؤΥݥ
 *  : Ǿå */
double
get_min_cut (GRAPH_C * gc, MAO * mao, HEAP * heap)
{
  int i;
  double mc, c;

  mc = DBL_MAX;
  for (i = 0; i < gc->n - 1; i++)
    {
      /* ܽ */
      get_mao (0, gc, mao, heap);

      /* Ǿå */
      c = get_cut_size_c (mao->t1, gc);
      if (c < mc)
        mc = c;

      /*  */
      contract_node_c (mao->t2, mao->t1, gc);
    }

  return mc;
}


/* ͥõ */
void
depth_first_search (int v, int *n, int visit[], int order[], GRAPH_D * gd)
{
  EDGE_D *e;

  visit[v] = 1;
  for (e = gd->adjlist[v]; e != NULL; e = e->next)
    {
      if (visit[e->node] == 0)
        depth_first_search (e->node, n, visit, order, gd);
    }
  order[*n] = v;
  ++*n;
}


/* եȥݥ륽
 * ) visit  0 ǥꥢƤɬפޤ */
void
topological_sort (int s, int order[], int visit[], GRAPH_D * gd)
{
  int n;

  n = 0;
  depth_first_search (s, &n, visit, order, gd);
}


/* Ϣʬ */
int
search_sc_component (int v, int *id, int visit[], GRAPH_D * gd, STACK * st,
                     SLIST * sl)
{
  int m, min, t;
  EDGE_D *e;

  ++*id;
  visit[v] = min = *id;
  push (v, st);

  for (e = gd->adjlist[v]; e != NULL; e = e->next)
    {
      if (visit[e->node] == 0)
        m = search_sc_component (e->node, id, visit, gd, st, sl);
      else
        m = visit[e->node];
      if (m < min)
        min = m;
    }

  if (min == visit[v])
    {
      while (1)
        {
          t = pop (st);
          visit[t] = gd->n;
          if (t == v)
            break;
          union_slist (t, v, sl);
        }
    }

  return min;
}


/* դǽդդ˥ԡ */
void
copy_into_graph_c (GRAPH * g, GRAPH_C * gc)
{
  int i;
  EDGE *e;

  for (i = 0; i < g->n; i++)
    {
      for (e = g->adjlist[i]; e != NULL; e = e->next)
        {
          if (e->node > i)
            insert_edge_c (i, e->node, e->weight, gc);
        }
    }
  simplification_c (gc);
}


/***************************************************************************/

int *order;             /* ȥݥ륽ȵ */
int *type;              /* MC-partition Σµ */
int *mpbuf;             /* ¿ŪХåե(MultiPurpose BUFfer) */
int *oldcut;            /* old cuts  */
GRAPH g;
GRAPH_C gc, gcc, gt;
GRAPH_F gf;
GRAPH_D gd;
SLIST slf, slp;
MAO mao;
SLIST slc;
STACK st, str, stc;
MLIST ml;
HEAP heap;
double mc;


/* ܽμǾåȤˤʤޤǽ */
void
find_mc_node (int s)
{
  int i, j;

  while (gc.ne > 1)
    {
      /* ܽ */
      get_mao (s, &gc, &mao, &heap);

      /* ΥåͤǾåȤнλ */
      if (get_cut_size_c (mao.t1, &gc) == mc)
        break;

      /* 
       * []
       * (ֹ椬 g.n ʾ)˼(ֹ椬 g.n̤)
       * 󤵤ƤޤΤɤᡢֹ椬礭롣 */
      if (mao.t1 > mao.t2)
        i = mao.t1, j = mao.t2;
      else
        i = mao.t2, j = mao.t1;
      union_slist (i, j, &slc);
      contract_node_c (i, j, &gc);
      delete_from_mlist (i, &ml);
      oldcut[i] = oldcut[j] = 0;
    }
}


/* ǽդդե׻ѥդ˥ԡ */
void
copy_into_graph_f (void)
{
  int i, v;
  EDGE_C *e;

  for (i = 0; i < gc.ne; i++)
    delete_node_f (mao.nfo[i], &gf);
  for (i = 0; i < gc.ne - 1; i++)
    {
      v = mao.nfo[i];
      for (e = gc.adjlist[v]; e != NULL; e = e->next)
        {
          if (mao.ofn[e->node] > i)
            insert_directed_edge_f (e->node, v, e->weight, &gf);
        }
    }
}


/* ե׻
 * s, t : եή (s  t ˸äή) */
void
get_flow (int s, int t)
{
  int i, v;
  double f;
  EDGE_F *e;

  /* ե */
  for (i = 0; i < mao.no; i++)
    gf.flow[mao.nfo[i]] = 0;

  /* ׻եɽ */
  if (verbose >= 2)
    printf ("flow (%d ==> %d : %g):\n", s, t, mc);

  /* ե׻ */
  gf.flow[s] = +mc;
  gf.flow[t] = -mc;
  for (i = mao.no - 1; i > 0; i--)
    {
      v = mao.nfo[i];
      e = gf.adjlist[v];
      f = gf.flow[v];
      while (f != 0 && e != NULL)
        {
          if (e->weight >= fabs (f))
            {
              /* ޤνŤߤեʾξ */
              e->flow = f;
              f = 0;
            }
          else
            {
              /* ޤνŤߤե̤ξ */
              if (f > 0)
                e->flow = +e->weight;
              else
                e->flow = -e->weight;
              f -= e->flow;
            }
          gf.flow[e->node] += e->flow;
          if (verbose >= 2)
            {
              if (e->flow > 0)
                printf ("%d ==> %d : %g / %g\n", v, e->node, +e->flow,
                        e->weight);
              else
                printf ("%d <== %d : %g / %g\n", v, e->node, -e->flow,
                        e->weight);
            }
          e = e->next;
        }
    }
}


/* եޤήƤޤ, ξ˻ޤ󤷤
 *  */
void
get_contracted_graph (void)
{
  int i, j, v, nn, *visit = mpbuf, *nbuf = mpbuf;
  EDGE_F *e;

  /*  */
  for (i = 0; i < mao.no; i++)
    visit[mao.nfo[i]] = 0;
  init_part_slist (mao.no, mao.nfo, &slf);
  init_subgraph_d (mao.no, mao.nfo, &gd);

  /* եޤήƤʤ֤ξλޤ
   * ޤήƤ֤ˤϥեȵλޤ */
  for (i = mao.no - 1; i > 0; i--)
    {
      v = mao.nfo[i];
      for (e = gf.adjlist[v]; e != NULL; e = e->next)
        {
          if (e->flow == e->weight)
            insert_directed_edge_d (e->node, v, e->weight, &gd);
          else if (e->flow == -e->weight)
            insert_directed_edge_d (v, e->node, e->weight, &gd);
          else
            insert_edge_d (v, e->node, e->weight, &gd);
        }
    }

  /* Ϣʬ򸡺󤹤٤ */
  for (i = mao.no - 1; i >= 0; i--)
    {
      v = mao.nfo[i];
      if (visit[v] != 0)
        continue;
      nn = 0;
      search_sc_component (v, &nn, visit, &gd, &stc, &slf);
    }

  /*  */
  for (i = mao.no - 1; i >= 0; i--)
    {
      nn = get_member (mao.nfo[i], nbuf, &slf);
      if (nn <= 1)
        continue;
      contract_nodes_d (nn, nbuf, &gd);

      if (oldcut[nbuf[0]] == 1)
        oldcut[nbuf[0]] = 2;
    }
}


/* MC-Partition ׻ */
void
get_mc_partition (void)
{
  int i, t, v, head, *flag = mpbuf;
  double c;
  EDGE_D *e;

  for (i = 0; i < gd.ne; i++)
    flag[order[i]] = 0;
  init_part_slist (gd.ne, order, &slp);

  /* ȣ¤ʬ */
  type[order[0]] = type[order[gd.ne - 1]] = 'B';
  for (i = 1; i < gd.ne - 1; i++)
    {
      v = order[i];
      if (get_cut_size_d (v, &gd) == mc)
        type[v] = 'A';
      else
        type[v] = 'B';
    }

  head = order[0];
  for (i = 1; i < gd.ne; i++)
    {
      v = order[i];

      if (type[v] == 'B')
        {
          /* ¤ξ */
          if (type[head] == 'B')
            /* ¤ξ */
            union_slist (v, head, &slp);
          else
            {
              /* ξ */
              t = head;
              while (t != NIL)
                {
                  flag[t] = 0;
                  t = slp.next[t];
                }
              head = v;
            }
        }
      else
        {
          /* ξ */
          flag[v] = 1;
          if (type[v] == 'B')
            /* ¤ξ */
            head = v;
          else
            {
              /* ξ */

              /* Σ v ½ΥåȤͤ׻ */
              c = mc;
              for (e = gd.adjlist[v]; e != NULL; e = e->next)
                {
                  if (flag[e->node] == 0)
                    c += e->weight;
                  else
                    c -= e->weight;
                }
              for (e = gd.revlist[v]; e != NULL; e = e->next)
                {
                  if (flag[e->node] == 0)
                    c += e->weight;
                  else
                    c -= e->weight;
                }

              if (c == mc)
                /* Ʊ˴ޤޤ */
                union_slist (v, head, &slp);
              else
                {
                  /* 㤦˴ޤޤ */
                  t = head;
                  while (t != NIL)
                    {
                      flag[t] = 0;
                      t = slp.next[t];
                    }
                  head = v;
                }
            }
        }
    }
}


/* վȥޥåԥ */
void
set_mapping (int s, int t)
{
  int ss;

  while (s != NIL)
    {
      delete_from_mlist (s, &ml);
      insert_into_mlist (s, t, &ml);
      for (ss = s; ss != NIL; ss = slc.next[ss])
        ml.dest[ss] = t;
      s = slf.next[s];
    }
}


/* 2-cycle-contraction ¹Ԥ */
void
two_cycle_contraction (int empty)
{
  EDGE_C *e;

  for (e = gcc.adjlist[empty]; e != NULL; e = e->next)
    {
      if (e->weight == 2)
        {
          contract_node_c (empty, e->node, &gcc);
          push (empty, &st);
          break;
        }
    }
}


/* (s,t)-cactus ¤ */
void
get_st_cactus (void)
{
  int i, v, prev, next, empty, u;

  prev = NIL;
  for (i = 0; i < gd.ne; i++)
    {
      v = order[i];
      if (!is_head (v, &slp))
        continue;

      empty = pop (&st);
      restore_node_c (empty, &gcc);
      u = pop (&st);
      restore_node_c (u, &gcc);
      set_mapping (v, u);
      if (type[v] == 'A')
        {
          /* ξ */
          insert_edge_c (prev, u, 1, &gcc);
          insert_edge_c (prev, empty, 1, &gcc);
          while (slp.next[v] != NIL)
            {
              next = pop (&st);
              restore_node_c (next, &gcc);
              insert_edge_c (u, next, 1, &gcc);
              v = slp.next[v];
              u = next;
              set_mapping (v, u);
            }
          insert_edge_c (u, empty, 1, &gcc);
          two_cycle_contraction (prev);
          prev = empty;
        }
      else
        {
          /* ¤ξ */
          if (prev != NIL)
            insert_edge_c (prev, u, 2, &gcc);
          while (slp.next[v] != NIL)
            {
              next = pop (&st);
              restore_node_c (next, &gcc);
              insert_edge_c (u, next, 2, &gcc);
              v = slp.next[v];
              u = next;
              set_mapping (v, u);
            }
          insert_edge_c (u, empty, 2, &gcc);
          if (prev != NIL)
            two_cycle_contraction (prev);
          prev = empty;
        }
    }
  two_cycle_contraction (prev);
}


/* old cuts ɤå
 *  : 0:old cut ʳޤ, 1: old cut */
int
check_oldcut (void)
{
  if (gd.ne == 2)
    {
      if (oldcut[order[0]] == 1 || oldcut[order[1]] == 1)
        return 1;
      else
        return 0;
    }
  if (gd.ne == 3)
    {
      if (oldcut[order[0]] == 1
          && (oldcut[order[1]] == 1 || type[order[1]] == 'B')
          && oldcut[order[2]] == 1)
        return 1;
      else
        return 0;
    }
  return 0;
}


#if REPLICATE != 0
/* դʣ */
void
replicate_graph_c (GRAPH_C * gs, GRAPH_C * gt)
{
  int i;
  EDGE_C *e;

  for (i = 0; i < gs->n; i++)
    {
      if (gs->state[i] == DELETED)
        {
          delete_node_c (i, gt);
          continue;
        }
      for (e = gs->adjlist[i]; e != NULL; e = e->next)
        {
          if (e->node > i)
            insert_edge_c (i, e->node, e->weight, gt);
        }
    }
}
#endif


int max_depth = 0;      /* Ƶκ翼 */
int num_recur = 0;      /* Ƶβ */


/* Ƶ롼 */
void
recur (int depth)
{
  int i, j, s, t, nn, v, joint, newnode, ne;
#if REPLICATE != 0
  GRAPH_C gr;
#endif

  /* Ƶ٤ȺƵ׻ */
  if (depth > max_depth)
    max_depth = depth;
  num_recur++;

  if (verbose >= 2)
    printf ("######## depth: %d ########\n\n", depth);

START:
  if (verbose >= 2)
    {
      printf ("input gc:\n");
      print_graph_c (&gc);
    }

  /* ǾåȤΣ */
  find_mc_node (depth + gc.n / 2 - 1);
  if (gc.ne <= 1)
    {
      if (verbose >= 2)
        printf ("there is no minimum cut\n\n");
      return;
    }
  if (verbose >= 2)
    {
      printf ("gc:\n");
      print_graph_c (&gc);
      printf ("slc:\n");
      print_slist (&slc);
      printf ("MAO:\n");
      for (i = 0; i < mao.no; i++)
        printf ("%d ", mao.nfo[i]);
      printf ("\n\n");
    }

  /* ե׻ѥպ */
  copy_into_graph_f ();
  if (verbose >= 3)
    {
      printf ("gf:\n");
      print_graph_f (&gf);
    }

  /* ե׻ */
  s = mao.t1;
  t = gf.adjlist[s]->node;
  get_flow (s, t);
  if (verbose >= 2)
    printf ("\n");

  /*  */
  get_contracted_graph ();
  if (verbose >= 3)
    {
      printf ("gd:\n");
      print_graph_d (&gd);
    }
  if (verbose >= 2)
    {
      printf ("slf:\n");
      print_slist (&slf);
    }

  /* եȥݥ륽 */
  for (i = 0; i < mao.no; i++)
    mpbuf[mao.nfo[i]] = 0;
  topological_sort (find_head (t, &slf), order, mpbuf, &gd);
  if (verbose >= 2)
    {
      printf ("reverse topological sort:\n");
      for (i = 0; i < gd.ne; i++)
        printf ("%d ", order[i]);
      printf ("\n\n");
    }

  /* MC-partition */
  get_mc_partition ();
  if (verbose >= 3)
    {
      printf ("slp:\n");
      print_slist (&slp);
    }
  if (verbose >= 2)
    {
      printf ("MC-partition:\n");
      for (i = 0; i < gd.ne; i++)
        {
          if (is_head (order[i], &slp))
            printf ("%c ", type[order[i]]);
          else
            printf ("- ");
        }
      printf ("\n\n");
    }

  /* old cut åoldcut[] 򸵤᤹ */
  j = check_oldcut ();
  for (i = 0; i < gd.ne; i++)
    {
      if (oldcut[order[i]] == 2)
        oldcut[order[i]] = 1;
    }

  /* ٤ old cut ξ */
  if (j == 1)
    {
      /* ʲʤ餳ʾǾåȤϸĤʤ */
      if (gc.ne <= 3)
        {
          if (verbose >= 2)
            printf ("all minimum cuts in gc are contained in old cuts\n\n");
          return;
        }

      if (verbose >= 2)
        printf ("######## gc = gc/{%d,%d} ########\n", s, t);
      /*
       * []
       * (ֹ椬 g.n ʾ)˼(ֹ椬 g.n̤)
       * 󤵤ƤޤΤɤᡢֹ椬礭롣 */
      if (s < t)
        i = s, s = t, t = i;
      contract_node_c (s, t, &gc);
      union_slist (s, t, &slc);
      delete_from_mlist (s, &ml);
      oldcut[s] = oldcut[t] = 0;
      goto START;
    }

  /* (s,t)-cactus  */
  get_st_cactus ();
  if (verbose >= 2)
    {
      printf ("(s,t)-cactus:\n");
      print_graph_c (&gcc);
      printf ("mapping:\n");
      for (i = 0; i < gc.n / 2; i++)
        printf ("%d ==> %d\n", i, ml.dest[i]);
      printf ("\n");
    }

  /* Ƶ */
  ne = gd.ne;
  for (i = 0; i < ne; i++)
    push (order[i], &str);
  for (i = 0; i < ne; i++)
    {
      v = pop (&str);
      joint = ml.dest[v];
      newnode = depth + gc.n / 2;
      nn = get_member_in_mlist (joint, mpbuf, &ml);
      if (nn <= 1)
        continue;

      /* ս */
#if REPLICATE == 0
      contract_nodes_c (nn, mpbuf, newnode, &gc, &gt);
#else
      create_graph_c (gc.n, &gr);
      replicate_graph_c (&gc, &gr);
      contract_nodes_ir_c (nn, mpbuf, newnode, &gc);
#endif

      ml.dest[newnode] = NIL;
      if (get_cut_size_c (newnode, &gc) == mc)
        oldcut[newnode] = 1;
      recur (depth + 1);
      oldcut[newnode] = 0;

      /* ս */
#if REPLICATE == 0
      uncontract_nodes_c (newnode, &gc, &gt);
#else
      init_graph_c (&gc);
      replicate_graph_c (&gr, &gc);
      free_graph_c (&gr);
#endif

      /*  */
      if (ml.dest[newnode] != NIL)
        {
          insert_edge_c (joint, ml.dest[newnode], 2, &gcc);
          contract_node_c (joint, ml.dest[newnode], &gcc);
          push (joint, &st);
        }

      /* newnode ̤Ѿ֤ˤ */
      delete_from_mlist (newnode, &ml);
#if 1
      separate_slist (newnode, &slc);
#endif
    }
}


/* ޥåԥ󥰤ˤ */
void
complete_mapping (void)
{
  int i, j;

  for (i = 0; i < g.n; i++)
    {
      if (!is_head (i, &slc))
        continue;
      for (j = slc.next[i]; j != NIL; j = slc.next[j])
        insert_into_mlist (j, ml.dest[i], &ml);
    }
}


/* ̤ͤ */
void
pack_cactus (void)
{
  int s, t;

  s = gcc.ne;
  t = 0;
  while (1)
    {
      while (t < gcc.ne && gcc.state[t] == EXISTING)
        t++;
      if (t >= gcc.ne)
        break;
      while (gcc.state[s] != EXISTING)
        s++;
      move_mlist (s, t, &ml);
      move_node_c (s, t, &gcc);
    }
}


/* ɽ */
void
get_cactus (void)
{
  int i;

  /*  */
  create_graph_c (2 * g.n, &gc);
#if REPLICATE == 0
  create_graph_c (2 * g.n, &gt);
#endif
  create_graph_f (2 * g.n, &gf);
  create_graph_d (2 * g.n, &gd);
  create_mao (2 * g.n, &mao);
  create_slist (2 * g.n, &slc);
  create_slist (2 * g.n, &slf);
  create_slist (2 * g.n, &slp);
  create_stack (gcc.n, &st);
  create_stack (gcc.n, &str);
  create_stack (2 * g.n, &stc);
  create_mlist (gcc.n, 2 * g.n, &ml);
#if HEAP_TYPE == 'd'
  create_heap (2 * g.n, 1 + (g.m + 1) / g.n, &heap);
#else
  create_heap (2 * g.n, &heap);
#endif
  order = (int *) my_malloc (2 * g.n * sizeof (int));
  type = (int *) my_malloc (2 * g.n * sizeof (int));
  mpbuf = (int *) my_malloc (2 * g.n * sizeof (int));
  oldcut = (int *) my_calloc (2 * g.n, sizeof (int));

  /*  */
  copy_into_graph_c (&g, &gc);
  for (i = g.n; i < 2 * g.n; i++)
    delete_node_c (i, &gc);
  for (i = 0; i < gcc.n; i++)
    delete_node_c (i, &gcc);
  for (i = gcc.n - 1; i >= 0; i--)
    push (i, &st);

  /* Ƶ롼ƤӽФ */
  recur (0);

  /* ޥåԥ󥰤ˤ */
#if 1
  complete_mapping ();
#endif

  /* ̤ͤ */
#if 1
  pack_cactus ();
#endif

  /* ɽ */
  if (verbose >= 1)
    {
      printf ("cactus:\n");
      print_graph_c (&gcc);
      printf ("mapping:\n");
      for (i = 0; i < g.n; i++)
        printf ("%d ==> %d\n", i, ml.dest[i]);
#if 0
      /* CHECK MLIST */
      for (i = 0; i < gcc.n; i++)
        {
          int j;
          printf ("%d: ", i);
          for (j = ml.head[i]; j != NIL; j = ml.next[j])
            printf (" - %d", j);
          printf ("\n");
        }
#endif
    }

  /*  */
  free_graph_c (&gc);
#if REPLICATE == 0
  free_graph_c (&gt);
#endif
  free_graph_f (&gf);
  free_graph_d (&gd);
  free_mao (&mao);
  free_slist (&slf);
  free_slist (&slp);
  free_slist (&slc);
  free_stack (&st);
  free_stack (&str);
  free_stack (&stc);
  free_mlist (&ml);
  free_heap (&heap);
  my_free (order, 2 * g.n);
  my_free (type, 2 * g.n);
  my_free (mpbuf, 2 * g.n);
  my_free (oldcut, 2 * g.n);
}


int
main (int argc, char *argv[])
{
  GRAPH_C gc;
  int c;
  int used_mem_chk = 0;
  int recur_chk = 0;

  set_program_name (argv[0]);

  /* ץ */
  while (1)
    {
      c = my_getopt (argc, argv, "v:mr");
      if (c == -1)
        break;

      switch (c)
        {
        case 'v':
          sscanf (optarg, "%d", &verbose);
          break;
        case 'm':
          used_mem_chk = 1;
          break;
        case 'r':
          recur_chk = 1;
          break;
        default:
          return EXIT_FAILURE;
        }
    }

  /*  */
  input_graph (&g);
  if (verbose >= 2)
    {
      printf ("g:\n");
      print_graph (&g);
    }

  /* ǽդպ */
  create_graph_c (g.n, &gc);
  copy_into_graph_c (&g, &gc);
  if (verbose >= 3)
    {
      printf ("gc:\n");
      print_graph_c (&gc);
    }

  /* Ǿåȷ׻ */
  create_mao (g.n, &mao);
#if HEAP_TYPE == 'd'
  create_heap (g.n, 1 + (g.m + 1) / g.n, &heap);
#else
  create_heap (g.n, &heap);
#endif
  mc = get_min_cut (&gc, &mao, &heap);
  if (verbose >= 1)
    printf ("minimum cut size = %g\n\n", mc);
  free_graph_c (&gc);
  free_mao (&mao);
  free_heap (&heap);
  if (mc == 0)
    return EXIT_SUCCESS;

  /* ɽ */
  create_graph_c (4 * g.n, &gcc);
  get_cactus ();
  free_graph (&g);
  free_graph_c (&gcc);

  if (used_mem_chk)
    printf ("max used memory: %d kbyte\n", max_alloc_size / 1024);

  if (recur_chk)
    printf ("number of revursive call: %d, max depth: %d\n",
            num_recur, max_depth);

  return EXIT_SUCCESS;
}
