/****************************************************************************
 *
 *  fheap.c : եܥʥåҡ׽
 *
 *  Programmed by Shuji Nakamura  Time-stamp: <2001-05-27>
 *
 ***************************************************************************/


#include <math.h>       /* log(), ceil() */
#include "fheap.h"
#include "mymalloc.h"



/* եܥʥåҡ׺
 * n : ǿ
 * h : եܥʥåҡ׹¤ΤؤΥݥ */
void
create_fheap (int n, FHEAP * h)
{
  h->n = n;
  h->max_rank = 1 + (int) ceil (1.5 * log (n) / log (2));
  h->key = (double *) my_malloc (n * sizeof (double));
  h->parent = (int *) my_malloc (n * sizeof (int));
  h->child = (int *) my_malloc (n * sizeof (int));
  h->prev = (int *) my_malloc (n * sizeof (int));
  h->next = (int *) my_malloc (n * sizeof (int));
  h->rank = (int *) my_malloc (n * sizeof (int));
  h->num_cut = (int *) my_malloc (n * sizeof (int));
  h->rank_buf = (int *) my_malloc (h->max_rank * sizeof (int));

  /*  */
  h->max = NIL;
}


/* եܥʥåҡ׽
 * h : եܥʥåҡ׹¤ΤؤΥݥ */
void
init_fheap (FHEAP * h)
{
  h->max = NIL;
}


/* եܥʥåҡײ
 * h : եܥʥåҡ׹¤ΤؤΥݥ */
void
free_fheap (FHEAP * h)
{
  my_free (h->key, h->n);
  my_free (h->parent, h->n);
  my_free (h->child, h->n);
  my_free (h->prev, h->n);
  my_free (h->next, h->n);
  my_free (h->rank, h->n);
  my_free (h->num_cut, h->n);
  my_free (h->rank_buf, h->max_rank);
}


/* next  node 
 * node, next : ֹ
 * h : եܥʥåҡ׹¤ΤؤΥݥ */
static void
insert_node (int node, int next, FHEAP * h)
{
  h->next[node] = next;
  h->prev[node] = h->prev[next];
  h->prev[next] = node;
  h->next[h->prev[node]] = node;
}


/* node 
 * node : ֹ
 * h : եܥʥåҡ׹¤ΤؤΥݥ */
static void
delete_node (int node, FHEAP * h)
{
  h->next[h->prev[node]] = h->next[node];
  h->prev[h->next[node]] = h->prev[node];
}


/*  key ֹ֤
 * h : եܥʥåҡ׹¤ΤؤΥݥ */
int
find_max_fheap (FHEAP * h)
{
  return h->max;
}


/* ֹ node Ǥեܥʥåҡפ
 * node : ֹ
 * key : 
 * h : եܥʥåҡ׹¤ΤؤΥݥ */
void
insert_fheap (int node, double key, FHEAP * h)
{
  h->key[node] = key;
  h->rank[node] = 0;
  h->num_cut[node] = 0;
  h->parent[node] = NIL;
  h->child[node] = NIL;
  if (h->max == NIL)
    {
      h->max = node;
      h->next[node] = h->prev[node] = node;
    }
  else
    {
      insert_node (node, h->max, h);
      if (h->key[h->max] < h->key[node])
        h->max = node;
    }
}


/* node1, node2 ޤޤꥹȤ
 * node1, node2 : ֹ
 * h : եܥʥåҡ׹¤ΤؤΥݥ */
static void
unite_list (int node1, int node2, FHEAP * h)
{
  int prev;

  if (node2 != NIL)
    {
      prev = h->prev[node1];
      h->next[prev] = node2;
      h->prev[node1] = h->prev[node2];
      h->prev[node2] = prev;
      h->next[h->prev[node1]] = node1;
    }
}


/* node Ʊ󥯤ڤ󥯤
 * node : ֹ
 * h : եܥʥåҡ׹¤ΤؤΥݥ */
static void
link (int node, FHEAP * h)
{
  int node2, t, i;

  for (i = h->rank[node]; h->rank_buf[i] != NIL; i++)
    {
      node2 = h->rank_buf[i];
      h->rank_buf[i] = NIL;

      /* key 礭ۤ node ˤ */
      if (h->key[node] < h->key[node2])
        t = node, node = node2, node2 = t;

      /* node2 == h->max ʤ h->key[node2] == h->key[node] ʤΤ
       * h->max  node ѹ */
      if (node2 == h->max)
        h->max = node;

      /* node2  node λҤˤ */
      delete_node (node2, h);
      h->parent[node2] = node;
      h->rank[node] = i + 1;
      if (i != 0)
        insert_node (node2, h->child[node], h);
      else
        {
          h->next[node2] = h->prev[node2] = node2;
          h->child[node] = node2;
        }
    }

  h->rank_buf[i] = node;
}


/*  key 
 * h : եܥʥåҡ׹¤ΤؤΥݥ */
void
delete_max_fheap (FHEAP * h)
{
  int node, next, last, i;

  /*  */
  for (i = 0; i < h->max_rank; i++)
    h->rank_buf[i] = NIL;

  /*  */
  if (h->next[h->max] == h->max)
    /* Ĥʤ餽λҤ h->max Ȥ */
    h->max = h->child[h->max];
  else
    {
      /* h->max λҤ򼡤 h->max ˷礹 */
      delete_node (h->max, h);
      unite_list (h->next[h->max], h->child[h->max], h);
      h->max = h->next[h->max];
    }

  /* ʤʤä齪λ */
  if (h->max == NIL)
    return;

  /* 󥯥ƥå */
  last = h->prev[h->max];
  next = h->max;
  do
    {
      node = next;
      next = h->next[node];
      h->parent[node] = NIL;

      /* h->max ѹ */
      if (h->key[h->max] < h->key[node])
        h->max = node;

      /* Ʊ󥯤ڤ */
      if (h->rank_buf[h->rank[node]] == NIL)
        h->rank_buf[h->rank[node]] = node;
      else
        link (node, h);
    }
  while (node != last);
}


/* node 򥫥å
 * node : åȤ
 * parent : node ο
 * h : եܥʥåҡ׹¤ΤؤΥݥ */
static void
cut (int node, int parent, FHEAP * h)
{
  /* node ڤ */
  h->rank[parent]--;
  if (h->rank[parent] == 0)
    /* 󥯤ˤʤäҤϤʤ */
    h->child[parent] = NIL;
  else
    {
      if (node == h->child[parent])
        h->child[parent] = h->next[node];
      delete_node (node, h);
    }

  /* node 򺬤 */
  h->parent[node] = NIL;
  insert_node (node, h->max, h);

  /* åȲû */
  h->num_cut[parent]++;
}


/* ֹ node  key  delta ä
 * node : ֹ
 * delta : ä
 * h : եܥʥåҡ׹¤ΤؤΥݥ */
void
increase_key_fheap (int node, double delta, FHEAP * h)
{
  int parent;

  /* key ѹ */
  h->key[node] += delta;

  /* node οƤξ */
  if (h->parent[node] == NIL)
    {
      if (h->key[h->max] < h->key[node])
        h->max = node;
      return;
    }

  /* åȤԤҡ׾ */
  parent = h->parent[node];
  if (h->key[parent] < h->key[node])
    {
      cut (node, parent, h);
      if (h->key[h->max] < h->key[node])
        h->max = node;

      /* åȥƥå */
      node = parent;
      parent = h->parent[node];
      while (parent != NIL && h->num_cut[node] == 2)
        {
          cut (node, parent, h);
          h->num_cut[node] = 0;
          node = parent;
          parent = h->parent[node];
        }
    }
}


#if 0
#include <stdio.h>
int
main (int argc, char *argv[])
{
  int i, buf[256];
  FHEAP h;

  create_fheap (256, &h);

  for (i = 1; i < argc; i++)
    sscanf (argv[i], "%d", buf + i);

  for (i = 1; i < argc; i++)
    insert_fheap (i, buf[i], &h);

  increase_key_fheap (3, 1000000, &h);

  for (i = 1; i < argc; i++)
    {
      printf ("%d(key:%g) ", find_max_fheap (&h), h.key[find_max_fheap (&h)]);
      delete_max_fheap (&h);
    }
  printf ("\n");
}
#endif
