/****************************************************************************
 *
 *  graph.c : ս
 *
 *  Programmed by Shuji Nakamura  Time-stamp: <2001-07-11>
 *
 ***************************************************************************/


#include <stdio.h>      /* NULL, stdin, printf(), sscanf(), fgets() */
#include <string.h>     /* strpbrk() */
#include "graph.h"
#include "error.h"
#include "mymalloc.h"


/* պ (Τߤǻޤ̵֤˽)
 * n : 
 * g : չ¤ΤؤΥݥ */
void
create_graph (int n, GRAPH * g)
{
  g->n = n;
  g->m = 0;
  g->adjlist = (EDGE **) my_calloc (n, sizeof (EDGE *));
}


/* դޤ̵֤˽
 * g : չ¤ΤؤΥݥ */
void
init_graph (GRAPH * g)
{
  int i;
  EDGE *e, *t;

  for (i = 0; i < g->n; i++)
    {
      e = g->adjlist[i];
      g->adjlist[i] = NULL;
      while (e != NULL)
        {
          t = e;
          e = e->next;
          my_free (t, 1);
        }
    }
  g->m = 0;
}


/* դ
 * g : չ¤ΤؤΥݥ */
void
free_graph (GRAPH * g)
{
  init_graph (g);
  my_free (g->adjlist, g->n);
}


/* դλޤ
 * s : 
 * t : 
 * w : Ť
 * g : չ¤ΤؤΥݥ */
static void
insert_directed_edge (int s, int t, double w, GRAPH * g)
{
  EDGE *e;

  e = (EDGE *) my_malloc (sizeof (EDGE));

  e->next = g->adjlist[s];
  e->prev = NULL;
  e->node = t;
  e->weight = w;
  if (e->next != NULL)
    e->next->prev = e;
  g->adjlist[s] = e;
}


/* դξλޤ
 * v1, v2 : ޤξüֹ
 * w : ޤνŤ
 * g : չ¤ΤؤΥݥ */
void
insert_edge (int v1, int v2, double w, GRAPH * g)
{
  insert_directed_edge (v1, v2, w, g);
  insert_directed_edge (v2, v1, w, g);
  g->m++;
}


/* դɸϤɤ߹
 * g : չ¤ΤؤΥݥ */
void
input_graph (GRAPH * g)
{
  char buf[256], *p;
  int i, j, n, done, len;
  double w;

  /* Ƚ */
  do
    {
      if (fgets (buf, sizeof buf, stdin) == NULL)
        error ("no input data");
    }
  while (buf[0] == '#');

  /* ɤ߹ */
  done = sscanf (buf, "%d", &n);
  if (done != 1 || n <= 0)
    error ("number of node must be a positive integer");

  /* պ */
  create_graph (n, g);

  /* ޥǡɤ߹ */
  while (fgets (buf, sizeof buf, stdin) != NULL)
    {
      /* Ƭʸ '#' ʤ饳ȤȤߤʤ */
      if (buf[0] == '#')
        continue;

      p = buf;
      while (1)
        {
          /* Ƭοʳʸѥ */
          p = strpbrk (p, "0123456789");
          if (p == NULL)
            break;

          /* Ѵ */
          done = sscanf (p, "%d%*[^0-9]%d%*[^0-9]%lf%n", &i, &j, &w, &len);
          if (done < 3)
            error ("too few parameter of edge");
          if (i < 0 || i >= n || j < 0 || j >= n || w <= 0.0)
            error ("bad edge data");

          insert_edge (i, j, w, g);
          p += len;
        }
    }
}


/* դܥꥹȤι¤ɽ
 * g : չ¤ΤؤΥݥ */
void
print_graph (GRAPH * g)
{
  int i;
  EDGE *e;

  for (i = 0; i < g->n; i++)
    {
      printf ("%d:", i);
      for (e = g->adjlist[i]; e != NULL; e = e->next)
        printf (" - %d", e->node);
      printf ("\n");
    }
  printf ("\n");
}


/***************************************************************************/


/* եդպ (Τߤǻޤ̵֤˽)
 * n : 
 * g : եեչ¤ΤؤΥݥ */
void
create_graph_f (int n, GRAPH_F * g)
{
  g->n = n;
  g->adjlist = (EDGE_F **) my_calloc (n, sizeof (EDGE_F *));
  g->flow = (double *) my_malloc (n * sizeof (double));
}


/* եդդޤ̵֤˽
 * g : եդչ¤ΤؤΥݥ */
void
init_graph_f (GRAPH_F * g)
{
  int i;
  EDGE_F *e, *t;

  for (i = 0; i < g->n; i++)
    {
      e = g->adjlist[i];
      g->adjlist[i] = NULL;
      while (e != NULL)
        {
          t = e;
          e = e->next;
          my_free (t, 1);
        }
    }
}


/* եդդ
 * g : եդչ¤ΤؤΥݥ */
void
free_graph_f (GRAPH_F * g)
{
  init_graph_f (g);
  my_free (g->adjlist, g->n);
  my_free (g->flow, g->n);
}


/* եդդ
 * v : ֹ
 * g : եդչ¤ΤؤΥݥ */
void
delete_node_f (int v, GRAPH_F * g)
{
  EDGE_F *e, *t;

  e = g->adjlist[v];
  g->adjlist[v] = NULL;
  while (e != NULL)
    {
      t = e;
      e = e->next;
      my_free (t, 1);
    }
}


/* եդդλޤ
 * s : 
 * t : 
 * w : Ť
 * g : եեչ¤ΤؤΥݥ */
void
insert_directed_edge_f (int s, int t, double w, GRAPH_F * g)
{
  EDGE_F *e;

  e = (EDGE_F *) my_malloc (sizeof (EDGE_F));
  e->next = g->adjlist[s];
  e->prev = NULL;
  e->node = t;
  e->weight = w;
  e->flow = 0;
  if (e->next != NULL)
    e->next->prev = e;
  g->adjlist[s] = e;
}


/* եդդܥꥹȤι¤ɽ
 * g : եդչ¤ΤؤΥݥ */
void
print_graph_f (GRAPH_F * g)
{
  int i;
  EDGE_F *e;

  for (i = 0; i < g->n; i++)
    {
      printf ("%d:", i);
      for (e = g->adjlist[i]; e != NULL; e = e->next)
        printf (" - %d", e->node);
      printf ("\n");
    }
  printf ("\n");
}


/***************************************************************************/


/* ꥹȽ
 * g : ǽդչ¤ΤؤΥݥ */
static void
init_node_list_c (GRAPH_C * g)
{
  int i;

  for (i = 0; i < g->n - 1; i++)
    {
      g->next[i] = i + 1;
      g->prev[i + 1] = i;
    }
  g->ehead = 0;
  g->next[g->n - 1] = g->prev[0] = g->dhead = NIL;
}


/* ǽդպ (Τߤǻޤ̵֤˽)
 * n : 
 * g : ǽդչ¤ΤؤΥݥ */
void
create_graph_c (int n, GRAPH_C * g)
{
  int i;

  g->n = g->ne = n;
  g->adjlist = (EDGE_C **) my_calloc (n, sizeof (EDGE_C *));
  g->state = (int *) my_calloc (n, sizeof (int));
  g->wbuf = (double *) my_calloc (n, sizeof (double));
  g->next = (int *) my_malloc (n * sizeof (int));
  g->prev = (int *) my_malloc (n * sizeof (int));

  init_node_list_c (g);
}


/* ޺
 * g : ǽդչ¤ΤؤΥݥ */
static void
delete_all_edge_c (GRAPH_C * g)
{
  int i;
  EDGE_C *e, *t;

  for (i = 0; i < g->n; i++)
    {
      e = g->adjlist[i];
      g->adjlist[i] = NULL;
      while (e != NULL)
        {
          t = e;
          e = e->next;
          my_free (t, 1);
        }
    }
}


/* ǽդդޤ̵֤˽
 * g : ǽդչ¤ΤؤΥݥ */
void
init_graph_c (GRAPH_C * g)
{
  int i;

  for (i = 0; i < g->n; i++)
    g->state[i] = EXISTING;
  delete_all_edge_c (g);
  init_node_list_c (g);
  g->ne = g->n;
}


/* ǽդդ
 * g : ǽդչ¤ΤؤΥݥ */
void
free_graph_c (GRAPH_C * g)
{
  delete_all_edge_c (g);
  my_free (g->adjlist, g->n);
  my_free (g->state, g->n);
  my_free (g->wbuf, g->n);
  my_free (g->next, g->n);
  my_free (g->prev, g->n);
}


/* ǽդդλޤ
 * s : 
 * t : 
 * w : Ť
 * g : ǽդչ¤ΤؤΥݥ
 *  : ޹¤ΤؤΥݥ */
static EDGE_C *
insert_directed_edge_c (int s, int t, double w, GRAPH_C * g)
{
  EDGE_C *e;

  e = (EDGE_C *) my_malloc (sizeof (EDGE_C));

  e->next = g->adjlist[s];
  e->prev = NULL;
  e->node = t;
  e->weight = w;
  e->reverse = NULL;
  if (e->next != NULL)
    e->next->prev = e;
  g->adjlist[s] = e;
  return e;
}


/* ǽդդξλޤ
 * v1, v2 : ޤξüֹ
 * w : ޤνŤ
 * g : ǽդչ¤ΤؤΥݥ */
void
insert_edge_c (int v1, int v2, double w, GRAPH_C * g)
{
  EDGE_C *e1, *e2;

  e1 = insert_directed_edge_c (v1, v2, w, g);
  e2 = insert_directed_edge_c (v2, v1, w, g);
  e1->reverse = e2;
  e2->reverse = e1;
}


/* ǽդդλޤ
 * v : ޹¤Τ°ܥꥹȤֹ
 * e : ޹¤ΤؤΥݥ
 * g : ǽդչ¤ΤؤΥݥ */
void
delete_edge_c (int v, EDGE_C * e, GRAPH_C * g)
{
  if (e->prev != NULL)
    e->prev->next = e->next;
  else
    g->adjlist[v] = e->next;
  if (e->next != NULL)
    e->next->prev = e->prev;

  my_free (e, 1);
}


/* Ƭ *s ꥹ v Ƭ *t ꥹȤ˰ư
 * v : ưֹ
 * s : ꥹȤƬֹؤΥݥ
 * t : ưꥹȤƬֹؤΥݥ
 * g : ǽդչ¤ΤؤΥݥ */
static void
move_list_c (int v, int *s, int *t, GRAPH_C * g)
{
  int prev = g->prev[v], next = g->next[v];

  if (prev != NIL)
    g->next[prev] = next;
  else
    *s = next;
  if (next != NIL)
    g->prev[next] = prev;

  g->next[v] = *t;
  g->prev[v] = NIL;
  if (*t != NIL)
    g->prev[*t] = v;
  *t = v;
}


/* ǽդդ
 * v : ֹ
 * g : ǽդչ¤ΤؤΥݥ */
void
delete_node_c (int v, GRAPH_C * g)
{
  EDGE_C *e, *t;

  if (g->state[v] == DELETED)
    return;

  e = g->adjlist[v];
  g->adjlist[v] = NULL;
  while (e != NULL)
    {
      t = e;
      e = e->next;
      if (t->reverse != NULL)
        delete_edge_c (t->node, t->reverse, g);
      my_free (t, 1);
    }

  move_list_c (v, &g->ehead, &g->dhead, g);
  g->state[v] = DELETED;
  g->ne--;
}


/* 򸵤᤹
 * v : ֹ᤹
 * g : ǽդչ¤ΤؤΥݥ */
void
restore_node_c (int v, GRAPH_C * g)
{
  move_list_c (v, &g->dhead, &g->ehead, g);
  g->state[v] = EXISTING;
  g->ne++;
}


/* ư
 * s : ưֹ
 * t : ưֹ
 * g : ǽդչ¤ΤؤΥݥ */
void
move_node_c (int s, int t, GRAPH_C * g)
{
  EDGE_C *e;

  restore_node_c (t, g);
  for (e = g->adjlist[s]; e != NULL; e = e->next)
    insert_edge_c (t, e->node, e->weight, g);
  delete_node_c (s, g);
}


/* ǽդդΣ󤹤
 * s : ˺ֹ
 * t : ˻Ĥֹ
 * g : ǽդչ¤ΤؤΥݥ */
void
contract_node_c (int s, int t, GRAPH_C * g)
{
  EDGE_C *e;

  for (e = g->adjlist[t]; e != NULL; e = e->next)
    g->wbuf[e->node] = e->weight;

  for (e = g->adjlist[s]; e != NULL; e = e->next)
    {
      if (e->node != t)
        {
          if (g->wbuf[e->node] == 0)
            insert_edge_c (t, e->node, e->weight, g);
          g->wbuf[e->node] += e->weight;
        }
    }

  for (e = g->adjlist[t]; e != NULL; e = e->next)
    {
      e->weight = e->reverse->weight = g->wbuf[e->node];
      g->wbuf[e->node] = 0;
    }

  delete_node_c (s, g);
}


/* ǽդդ󤹤(ĵ(irreversible))
 * nn : 󤵤ʤο
 * nbuf : 󤵤ʤֹ
 * t : ֹ (̤)
 * g : ǽդչ¤ΤؤΥݥ */
void
contract_nodes_ir_c (int nn, int nbuf[], int t, GRAPH_C * g)
{
  int i, v, next;
  EDGE_C *e;

  /* 󤵤ʤޡդ */
  for (i = 0; i < nn; i++)
    g->state[nbuf[i]] = -1;

  /* ˻ޤղ */
  for (i = 0; i < nn; i++)
    {
      v = nbuf[i];
      for (e = g->adjlist[v]; e != NULL; e = e->next)
        {
          if (g->state[e->node] == -1)
            continue;

          if (g->wbuf[v] == 0)
            insert_edge_c (v, t, e->weight, g);
          g->wbuf[v] += e->weight;
        }
    }

  /* Ť */
  for (e = g->adjlist[t]; e != NULL; e = e->next)
    {
      e->weight = e->reverse->weight = g->wbuf[e->node];
      g->wbuf[e->node] = 0;
    }

  /* 󤵤줿 */
  v = g->ehead;
  while (v != NIL)
    {
      next = g->next[v];
      if (g->state[v] == -1)
        g->state[v] = EXISTING;
      else
        delete_node_c (v, g);
      v = next;
    }

  restore_node_c (t, g);
}


/* ǽդդ󤹤
 * nn : 󤵤ʤο
 * nbuf : 󤵤ʤֹ
 * t : ֹ (̤)
 * g : 󤹤륰դؤΥݥ
 * gt : ݻѥդؤΥݥ */
void
contract_nodes_c (int nn, int nbuf[], int t, GRAPH_C * g, GRAPH_C * gt)
{
  int i, v, next;
  EDGE_C *e;

  /* 󤵤ʤޡդ */
  for (i = 0; i < nn; i++)
    g->state[nbuf[i]] = -1;

  /* ˻ޤղ */
  for (i = 0; i < nn; i++)
    {
      v = nbuf[i];
      for (e = g->adjlist[v]; e != NULL; e = e->next)
        {
          if (g->state[e->node] == -1)
            continue;

          if (g->wbuf[v] == 0)
            insert_edge_c (v, t, e->weight, g);
          g->wbuf[v] += e->weight;

          /* ޤ */
          e->reverse->reverse = NULL;
          delete_edge_c (v, e, g);
        }
    }

  /* Ť */
  for (e = g->adjlist[t]; e != NULL; e = e->next)
    {
      e->weight = e->reverse->weight = g->wbuf[e->node];
      g->wbuf[e->node] = 0;
    }

  /* 󤵤줿򸫤 */
  v = g->ehead;
  while (v != NIL)
    {
      next = g->next[v];
      if (g->state[v] == -1)
        g->state[v] = EXISTING;
      else
        {
          insert_directed_edge_c (t, v, 0, gt);
          move_list_c (v, &g->ehead, &g->dhead, g);
          g->state[v] = DELETED;
          g->ne--;
        }
      v = next;
    }

  restore_node_c (t, g);
}


/* 򸵤᤹
 * t : ֹ
 * g : 󤹤륰դؤΥݥ
 * gt : ݻѥդؤΥݥ */
void
uncontract_nodes_c (int t, GRAPH_C * g, GRAPH_C * gt)
{
  EDGE_C *e, *et;

  for (et = gt->adjlist[t]; et != NULL; et = et->next)
    {
      for (e = g->adjlist[et->node]; e != NULL; e = e->next)
        {
          if (e->reverse == NULL)
            e->reverse =
              insert_directed_edge_c (e->node, et->node, e->weight, g);
        }
      move_list_c (et->node, &g->dhead, &g->ehead, g);
      g->state[et->node] = EXISTING;
      g->ne++;
    }
  delete_node_c (t, gt);
}


/*  v μΥåȤͤ
 * v : ֹ
 * g : ǽդչ¤ΤؤΥݥ */
double
get_cut_size_c (int v, GRAPH_C * g)
{
  double c;
  EDGE_C *e;

  c = 0;
  for (e = g->adjlist[v]; e != NULL; e = e->next)
    c += e->weight;
  return c;
}


/* դñ㥰դѴ
 * g : ǽդչ¤ΤؤΥݥ */
void
simplification_c (GRAPH_C * g)
{
  int i;
  EDGE_C *e;

  for (i = 0; i < g->n; i++)
    {
      for (e = g->adjlist[i]; e != NULL; e = e->next)
	if (e->node != i)
	  g->wbuf[e->node] = e->weight;
      for (e = g->adjlist[i]; e != NULL; e = e->next)
	{
	  if (e->node == i || g->wbuf[e->node] == 0)
	    delete_edge_c (i, e, g);
	  else
	    {
	      e->weight = e->reverse->weight = g->wbuf[e->node];
	      g->wbuf[e->node] = 0;
	    }
	}
    }
}


/* ǽդդܥꥹȤι¤ɽ
 * g : ǽդչ¤ΤؤΥݥ */
void
print_graph_c (GRAPH_C * g)
{
  int i;
  EDGE_C *e;

  for (i = 0; i < g->n; i++)
    {
      if (g->state[i] == DELETED)
        continue;
      printf ("%d:", i);
      for (e = g->adjlist[i]; e != NULL; e = e->next)
        printf (" - %d", e->node);
      printf ("\n");
    }
  printf ("\n");
}


/***************************************************************************/


/* ǽդͭպ (Τߤǻޤ̵֤˽)
 * n : 
 * g : ǽդͭչ¤ΤؤΥݥ */
void
create_graph_d (int n, GRAPH_D * g)
{
  g->n = g->ne = n;
  g->adjlist = (EDGE_D **) my_calloc (n, sizeof (EDGE_D *));
  g->revlist = (EDGE_D **) my_calloc (n, sizeof (EDGE_D *));
  g->wbuf = (double *) my_calloc (n, sizeof (double));
}


/* ǽդͭդޤ̵֤˽
 * g : ǽդͭչ¤ΤؤΥݥ */
void
init_graph_d (GRAPH_D * g)
{
  int i;
  EDGE_D *e, *t;

  for (i = 0; i < g->n; i++)
    {
      e = g->adjlist[i];
      g->adjlist[i] = NULL;
      while (e != NULL)
        {
          t = e;
          e = e->next;
          my_free (t, 1);
        }
      e = g->revlist[i];
      g->revlist[i] = NULL;
      while (e != NULL)
        {
          t = e;
          e = e->next;
          my_free (t, 1);
        }
    }
  g->ne = g->n;
}


/* ǽդͭդΰޤ̵֤˽
 * g : ǽդͭչ¤ΤؤΥݥ */
void
init_subgraph_d (int nn, int nbuf[], GRAPH_D * g)
{
  int i, v;
  EDGE_D *e, *t;

  for (i = 0; i < nn; i++)
    {
      v = nbuf[i];
      e = g->adjlist[v];
      g->adjlist[v] = NULL;
      while (e != NULL)
        {
          t = e;
          e = e->next;
          my_free (t, 1);
        }
      e = g->revlist[v];
      g->revlist[v] = NULL;
      while (e != NULL)
        {
          t = e;
          e = e->next;
          my_free (t, 1);
        }
    }
  g->ne = nn;
}


/* ǽդͭդ
 * g : ǽդͭչ¤ΤؤΥݥ */
void
free_graph_d (GRAPH_D * g)
{
  init_graph_d (g);
  my_free (g->adjlist, g->n);
  my_free (g->revlist, g->n);
  my_free (g->wbuf, g->n);
}


/* ǽդͭդλޤ
 * s : 
 * t : 
 * w : Ť
 * g : ǽդͭչ¤ΤؤΥݥ */
void
insert_directed_edge_d (int s, int t, double w, GRAPH_D * g)
{
  EDGE_D *es, *et;

  es = (EDGE_D *) my_malloc (sizeof (EDGE_D));
  et = (EDGE_D *) my_malloc (sizeof (EDGE_D));

  es->next = g->adjlist[s];
  es->prev = NULL;
  es->node = t;
  es->weight = w;
  es->pair = et;
  if (es->next != NULL)
    es->next->prev = es;
  g->adjlist[s] = es;

  et->next = g->revlist[t];
  et->prev = NULL;
  et->node = s;
  et->weight = w;
  et->pair = es;
  if (et->next != NULL)
    et->next->prev = et;
  g->revlist[t] = et;
}


/* ǽդͭդξλޤ
 * v1, v2 : ޤξüֹ
 * w : Ť
 * g : ǽդͭչ¤ΤؤΥݥ */
void
insert_edge_d (int v1, int v2, double w, GRAPH_D * g)
{
  insert_directed_edge_d (v1, v2, w, g);
  insert_directed_edge_d (v2, v1, w, g);
}


/* ǽդͭդλޤ
 * v : ޹¤Τ°ܥꥹ (adjlist) ֹ
 * e : ޹¤ΤؤΥݥ (adjlist ¦)
 * g : ǽդͭչ¤ΤؤΥݥ */
void
delete_edge_d (int v, EDGE_D * e, GRAPH_D * g)
{
  if (e->prev != NULL)
    e->prev->next = e->next;
  else
    g->adjlist[v] = e->next;
  if (e->next != NULL)
    e->next->prev = e->prev;

  v = e->node;
  e = e->pair;

  if (e->prev != NULL)
    e->prev->next = e->next;
  else
    g->revlist[v] = e->next;
  if (e->next != NULL)
    e->next->prev = e->prev;

  my_free (e->pair, 1);
  my_free (e, 1);
}


/* ǽդͭդ
 * v : ֹ
 * g : ǽդͭչ¤ΤؤΥݥ */
void
delete_node_d (int v, GRAPH_D * g)
{
  EDGE_D *e, *t;

  e = g->adjlist[v];
  while (e != NULL)
    {
      t = e;
      e = e->next;
      delete_edge_d (v, t, g);
    }

  e = g->revlist[v];
  while (e != NULL)
    {
      t = e;
      e = e->next;
      delete_edge_d (t->node, t->pair, g);
    }

  g->ne--;
}


/* ǽդͭդ󤹤
 * nn : 󤹤ο
 * nbuf : 󤹤ֹ (nbuf[0] ˻Ĥ)
 * g : ǽդͭչ¤ΤؤΥݥ */
void
contract_nodes_d (int nn, int nbuf[], GRAPH_D * g)
{
  int i, t = nbuf[0];
  EDGE_D *e;

  for (e = g->adjlist[t]; e != NULL; e = e->next)
    g->wbuf[e->node] = e->weight;
  for (i = 1; i < nn; i++)
    {
      for (e = g->adjlist[nbuf[i]]; e != NULL; e = e->next)
        {
          if (e->node == t)
            continue;
          if (g->wbuf[e->node] == 0)
            /* ) e->node 󤵤ξ,פʻޤ뤬,
             *      delete_node ǤλޤϽΤʤ */
            insert_directed_edge_d (t, e->node, e->weight, g);
          g->wbuf[e->node] += e->weight;
        }
    }
  for (e = g->adjlist[t]; e != NULL; e = e->next)
    {
      e->weight = e->pair->weight = g->wbuf[e->node];
      g->wbuf[e->node] = 0;
    }

  for (e = g->revlist[t]; e != NULL; e = e->next)
    g->wbuf[e->node] = e->weight;
  for (i = 1; i < nn; i++)
    {
      for (e = g->revlist[nbuf[i]]; e != NULL; e = e->next)
        {
          if (e->node == t)
            continue;
          if (g->wbuf[e->node] == 0)
            insert_directed_edge_d (e->node, t, e->weight, g);
          g->wbuf[e->node] += e->weight;
        }
    }
  for (e = g->revlist[t]; e != NULL; e = e->next)
    {
      e->weight = e->pair->weight = g->wbuf[e->node];
      g->wbuf[e->node] = 0;
    }

  for (i = 1; i < nn; i++)
    delete_node_d (nbuf[i], g);
}


/*  v μΥåȤͤ
 * v : ֹ
 * g : ǽդͭչ¤ΤؤΥݥ */
double
get_cut_size_d (int v, GRAPH_D * g)
{
  double c;
  EDGE_D *e;

  c = 0;
  for (e = g->adjlist[v]; e != NULL; e = e->next)
    c += e->weight;
  for (e = g->revlist[v]; e != NULL; e = e->next)
    c += e->weight;
  return c;
}


/* ǽդͭդܥꥹȤι¤ɽ
 * g : ǽդͭչ¤ΤؤΥݥ */
void
print_graph_d (GRAPH_D * g)
{
  int i;
  EDGE_D *e;

  for (i = 0; i < g->n; i++)
    {
      printf ("%d:>", i);
      for (e = g->adjlist[i]; e != NULL; e = e->next)
        printf (" - %d", e->node);
      printf ("\n");
    }
  for (i = 0; i < g->n; i++)
    {
      printf ("%d:<", i);
      for (e = g->revlist[i]; e != NULL; e = e->next)
        printf (" - %d", e->node);
      printf ("\n");
    }
  printf ("\n");
}
