/****************************************************************************
 *
 *  pheap.c : ڥ󥰥ҡ׽ʥƥǤˤĤ갷ա
 *
 *  Programmed by Shuji Nakamura  Time-stamp: <2001-05-27>
 *
 ***************************************************************************/


#include "mymalloc.h"
#include "pheap.h"


void
create_pheap (int n, PHEAP * h)
{
  h->n = n;
  h->key = my_malloc (n * sizeof (double));
  h->child = my_malloc (n * sizeof (int));
  h->prev = my_malloc (n * sizeof (int));
  h->next = my_malloc (n * sizeof (int));
  h->buf = my_malloc (n * sizeof (int));
  h->root = NIL;
}


void
init_pheap (PHEAP * h)
{
  h->root = NIL;
}


void
free_pheap (PHEAP * h)
{
  my_free (h->key, h->n);
  my_free (h->child, h->n);
  my_free (h->next, h->n);
  my_free (h->prev, h->n);
  my_free (h->buf, h->n);
}


static int
comparison_link (int node1, int node2, PHEAP * h)
{
  if (node2 == NIL)
    return node1;

  if (h->key[node1] >= h->key[node2])
    {
      h->prev[node2] = node1;
      h->next[node1] = h->next[node2];
      if (h->next[node1] != NIL)
        h->prev[h->next[node1]] = node1;
      h->next[node2] = h->child[node1];
      if (h->next[node2] != NIL)
        h->prev[h->next[node2]] = node2;
      h->child[node1] = node2;
      return node1;
    }
  else
    {
      h->prev[node2] = h->prev[node1];
      h->prev[node1] = node2;
      h->next[node1] = h->child[node2];
      if (h->next[node1] != NIL)
        h->prev[h->next[node1]] = node1;
      h->child[node2] = node1;
      return node2;
    }
}


void
insert_pheap (int node, double key, PHEAP * h)
{
  h->key[node] = key;
  h->child[node] = h->next[node] = h->prev[node] = NIL;
  if (h->root == NIL)
    h->root = node;
  else
    h->root = comparison_link (h->root, node, h);
}


void
increase_key_pheap (int node, double delta, PHEAP * h)
{
  /* ѹ */
  h->key[node] += delta;

  if (node == h->root)
    return;

  /*  */
  if (h->next[node] != NIL)
    h->prev[h->next[node]] = h->prev[node];
  if (h->child[h->prev[node]] == node)
    h->child[h->prev[node]] = h->next[node];
  else
    h->next[h->prev[node]] = h->next[node];

  /*  comparison-link */
  h->next[node] = NIL;
  h->root = comparison_link (h->root, node, h);
}


void
delete_max_pheap (PHEAP * h)
{
  int i, j, num, node;

  node = h->child[h->root];
  if (node == NIL || h->next[node] == NIL)
    {
      h->root = node;
      return;
    }

  for (num = 0; node != NIL; num++)
    {
      h->buf[num] = node;
      h->next[h->prev[node]] = NIL;
      node = h->next[node];
    }

  /* left-to-right pass */
  for (i = 0; i + 1 < num; i += 2)
    h->buf[i] = comparison_link (h->buf[i], h->buf[i + 1], h);

  /* right-to-left pass */
  for (i = (i == num) ? i - 2 : i; i >= 2; i -= 2)
    h->buf[i - 2] = comparison_link (h->buf[i - 2], h->buf[i], h);

  h->root = h->buf[0];
}


int
find_max_pheap (PHEAP * h)
{
  return h->root;
}


#if 0
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
int
main (void)
{
  PHEAP h;
  int i;

  srand(time(NULL));
  create_pheap (20, &h);

  for(i = 0; i < 20; i++)
    insert_pheap (i, rand()%100, &h);

  for(i = 0; i < 20; i++)
    increase_key_pheap (i, rand()%100, &h);

  for (i = 0; i < 20; i++)
    {
      printf ("%d(%g)\n", find_max_pheap (&h), h.key[find_max_pheap (&h)]);
      delete_max_pheap (&h);
    }
  printf ("\n");
}
#endif
