/****************************************************************************
 *
 *  slist.c : ꥹȽ
 *
 *  Programmed by Shuji Nakamura  Time-stamp: <2001-05-27>
 *
 ***************************************************************************/


#include <stdio.h>      /* printf() */
#include "slist.h"
#include "error.h"
#include "mymalloc.h"


void
create_slist (int n, SLIST * sl)
{
  int i;

  sl->n = n;
  sl->next = (int *) my_malloc (n * sizeof (int));
  sl->prev = (int *) my_malloc (n * sizeof (int));

  init_slist (sl);
}


void
free_slist (SLIST * sl)
{
  my_free (sl->next, sl->n);
  my_free (sl->prev, sl->n);
}


void
init_slist (SLIST * sl)
{
  int i;

  for (i = 0; i < sl->n; i++)
    {
      sl->next[i] = NIL;
      sl->prev[i] = i;
    }
}


void
init_part_slist (int n, int buf[], SLIST * sl)
{
  int i, j;

  for (i = 0; i < n; i++)
    {
      j = buf[i];
      sl->next[j] = NIL;
      sl->prev[j] = j;
    }
}


int
is_head (int i, SLIST * sl)
{
  return sl->next[sl->prev[i]] == NIL;
}


int
is_tail (int i, SLIST * sl)
{
  return sl->next[i] == NIL;
}


void
union_slist (int s, int t, SLIST * sl)
{
  int ps, pt;

  if (!is_head (s, sl) || !is_head (t, sl))
    error ("argument s and t must be heads");

  /* s ΥꥹȤ t ΥꥹȤղ */
  ps = sl->prev[s];
  pt = sl->prev[t];
  sl->prev[s] = pt;
  sl->prev[t] = ps;
  sl->next[pt] = s;
}


void
separate_slist (int i, SLIST * sl)
{
  int ni = sl->next[i], pi = sl->prev[i];

  if (!is_head (i, sl))
    sl->next[pi] = ni;
  if (!is_tail (i, sl))
    sl->prev[ni] = pi;
  sl->next[i] = NIL;
  sl->prev[i] = i;
}


int
find_head (int i, SLIST * sl)
{
  while (!is_head (i, sl))
    i = sl->prev[i];
  return i;
}


int
get_member (int i, int buf[], SLIST * sl)
{
  int j;

  if (!is_head (i, sl))
    return 0;

  for (j = 0; i != NIL; j++)
    {
      buf[j] = i;
      i = sl->next[i];
    }

  return j;
}


void
print_slist (SLIST * sl)
{
  int i, t;

  for (i = 0; i < sl->n; i++)
    {
      printf ("%d :", i);
      if (!is_head (i, sl))
        {
          printf (" ==> %d\n", find_head (i, sl));
          continue;
        }

      t = sl->next[i];
      while (t != NIL)
        {
          printf (" - %d", t);
          t = sl->next[t];
        }
      printf ("\n");
    }
  printf ("\n");
}
